#include <stdio.h>  // printf

#include "sensirion_common.h"
#include "sensirion_i2c_hal.h"
#include "sts4x_i2c.h"

/*
 * TO USE CONSOLE OUTPUT (PRINTF) YOU MAY NEED TO ADAPT THE INCLUDE ABOVE OR
 * DEFINE IT ACCORDING TO YOUR PLATFORM:
 * #define printf(...)
 */

int main(void) {
    int16_t error = 0;

    /**
     * select the proper i2c address for your sensor (defaults to 0x46)
     *
     * STS40-CD1B: ADDR_STS4X (I2C address: 0x46)
     * STS40-AD1B: ADDR_STS4X_ALT (I2C address: 0x44)
     * STS40-BD1B: ADDR_STS4X_ALT2 (I2C address: 0x45)
     */
    init_driver(ADDR_STS4X_ALT);

    uint32_t serial_number;
    error = sts4x_serial_number(&serial_number);
    if (error) {
        printf("Error executing sts4x_serial_number(): %i\n", error);
    } else {
        printf("Serial number: %u\n", serial_number);
    }

    // Start Measurement
    for (;;) {
        int32_t temperature;
        // Read Measurement
        error = sts4x_measure_high_precision(&temperature);
        if (error) {
            printf("Error executing sts4x_measure_high_precision(): %i\n",
                   error);
        } else {
            printf("Temperature: %.1f °C\n", temperature / 1000.0f);
        }

        sensirion_i2c_hal_sleep_usec(1000000);
    }

    return 0;
}